@@ -7,7 +7,8 @@ import {
   page,
   layoutReady,
   ElementHandle,
-  waitForScrollEnd
+  waitForScrollEnd,
+  waitUntilDisappears
 } from './dramaturg';
 import { IScenario } from './benchmark';
 
@@ -17,11 +18,13 @@ import type { MenuOpenScenarioOptions } from './types/_scenario-menu-open';
 import type { CompleterScenarioOptions } from './types/_scenario-completer';
 import type { SidebarsScenarioOptions } from './types/_scenario-sidebars';
 import type { ScrollScenarioOptions } from './types/_scenario-scroll';
+import type { DebuggerScenarioOptions } from './types/_scenario-debugger';
 
 import scenarioOptionsSchema from './schema/scenario-base.json';
 import scenarioMenuOpenOptionsSchema from './schema/scenario-menu-open.json';
 import scenarioTabOptionsSchema from './schema/scenario-tabs.json';
 import scenarioCompleterOptionsSchema from './schema/scenario-completer.json';
+import scenarioDebuggerOptionsSchema from './schema/scenario-debugger.json';
 import scenarioSidebarsSchema from './schema/scenario-sidebars.json';
 import scenarioScrollSchema from './schema/scenario-scroll.json';
 
@@ -142,8 +145,16 @@ export function insertText(
   });
 }
 
+interface IExtendedDebuggerScenarioOptions extends DebuggerScenarioOptions {
+  editor: 'Notebook';
+  path: null;
+}
+
 class SingleEditorScenario<
-  T extends CompleterScenarioOptions | ScrollScenarioOptions
+  T extends
+    | CompleterScenarioOptions
+    | ScrollScenarioOptions
+    | IExtendedDebuggerScenarioOptions
 > {
   constructor(protected jupyterApp: JupyterFrontEnd) {
     // no-op
@@ -240,12 +251,26 @@ export class CompleterScenario
       tokens.push('t');
       text = tokens.join('\n');
     }
-    await insertText(this.jupyterApp, text);
+    if (this.useNotebook && this.options.setup.setupCell) {
+      await insertText(this.jupyterApp, this.options.setup.setupCell);
+      await waitForKernelStatus(this.widget.node, 'idle');
+      await this.jupyterApp.commands.execute(
+        'notebook:run-cell-and-insert-below'
+      );
+      await insertText(this.jupyterApp, text);
+      await this.jupyterApp.commands.execute('notebook:enter-edit-mode');
+    } else {
+      await insertText(this.jupyterApp, text);
+    }
 
     if (!this.useNotebook) {
       // Scroll down a little bit to avoid out of view bug
-      this.editor!.querySelector('.CodeMirror-scroll')!.scrollBy({
-        top: 500,
+      // `.CodeMirror-scroll` is CM5, `.cm-scroller` is CM6
+      const scrollArea =
+        this.editor!.querySelector('.CodeMirror-scroll')! ||
+        this.editor!.querySelector('.cm-scroller')!;
+      scrollArea.scrollBy({
+        top: 20 * this.options.setup.tokenCount,
         left: 0,
         behavior: 'smooth'
       });
@@ -264,8 +289,16 @@ export class CompleterScenario
     if (this.useNotebook) {
       // TODO enter a specific cell, not the first cell?
       const handle = new ElementHandle(this.widget!.node);
-      const editor = await handle.$('.jp-Editor textarea');
-      await editor!.focus();
+      const editorSelector = document.querySelector('.cm-content')
+        ? '.cm-content'
+        : 'textarea';
+      const editor = this.options!.setup.setupCell
+        ? await handle.$(`.jp-Cell:nth-child(2) .jp-Editor ${editorSelector}`)
+        : await handle.$(`.jp-Editor ${editorSelector}`);
+      if (!editor) {
+        throw Error('Setup failed: cell editor could not be located');
+      }
+      await editor.focus();
     }
     await layoutReady();
     await page.press('Tab');
@@ -276,8 +309,12 @@ export class CompleterScenario
     // need to query for a completer with programatically set styles (which are
     // things like position (top/left/width/height) which are only present in the
     // active completer
-    await page.waitForSelector('.jp-Completer[style]', { state: 'attached' });
-    await page.waitForSelector('.jp-Completer[style]', { state: 'visible' });
+    await page.waitForSelector('.jp-Completer.jp-HoverBox[style]', {
+      state: 'attached'
+    });
+    await page.waitForSelector('.jp-Completer.jp-HoverBox[style]', {
+      state: 'visible'
+    });
     await layoutReady();
   }
 
@@ -289,6 +326,125 @@ export class CompleterScenario
   }
 }
 
+export class DebuggerScenario
+  extends SingleEditorScenario<IExtendedDebuggerScenarioOptions>
+  implements IScenario
+{
+  id = 'debugger';
+  name = 'Debugger';
+  configSchema = scenarioDebuggerOptionsSchema as any as JSONSchema7;
+
+  setOptions(options: DebuggerScenarioOptions): void {
+    super.setOptions({
+      ...options,
+      editor: 'Notebook',
+      path: null
+    });
+  }
+
+  private async _goToTop() {
+    if (!this.options) {
+      throw new Error('Setup failure');
+    }
+    for (let i = 0; i < this.options.codeCells.length + 1; i++) {
+      await this.jupyterApp.commands.execute('notebook:move-cursor-up');
+    }
+  }
+
+  async setupSuite(): Promise<void> {
+    await super.setupSuite();
+    if (!this.widget || !this.options) {
+      throw new Error('Parent setup failure');
+    }
+
+    for (const codeCell of this.options.codeCells) {
+      await insertText(this.jupyterApp, codeCell);
+      await this.jupyterApp.commands.execute('notebook:insert-cell-below');
+      await layoutReady();
+      await layoutReady();
+      await this.jupyterApp.commands.execute('notebook:enter-edit-mode');
+      await layoutReady();
+      await layoutReady();
+    }
+    await insertText(this.jupyterApp, '%reset -f');
+    await this._goToTop();
+
+    await waitForKernelStatus(this.widget.node, 'idle');
+    const handle = new ElementHandle(this.widget.node);
+    const bugIcon = await handle.waitForSelector(
+      'button[aria-disabled="false"][title="Enable Debugger"]',
+      {
+        state: 'attached'
+      }
+    );
+    await bugIcon.click();
+    await page.waitForSelector(`#${CSS.escape('jp-debugger-sidebar')}`, {
+      state: 'visible'
+    });
+    await handle.waitForSelector(
+      'button[aria-disabled="false"][title="Disable Debugger"]',
+      {
+        state: 'attached'
+      }
+    );
+    await layoutReady();
+
+    await page.waitForSelector('.jp-DebuggerVariables-body', {
+      state: 'attached'
+    });
+    await page.waitForSelector('.jp-DebuggerVariables-body', {
+      state: 'visible'
+    });
+  }
+
+  async run(): Promise<void> {
+    if (!this.widget || !this.options) {
+      throw new Error('Setup failure');
+    }
+
+    for (let i = 0; i < this.options.codeCells.length; i++) {
+      await this.jupyterApp.commands.execute(
+        'notebook:run-cell-and-select-next'
+      );
+      await waitForKernelStatus(this.widget.node, 'idle');
+      const expectedCount = this.options.expectedNumberOfVariables[i];
+      if (typeof expectedCount !== 'undefined') {
+        await page.waitForSelector(
+          `.jp-DebuggerVariables-body li:nth-child(${expectedCount + 2})`,
+          {
+            state: 'attached'
+          }
+        );
+      }
+      await layoutReady();
+    }
+  }
+
+  async cleanup(): Promise<void> {
+    if (!this.widget || !this.options) {
+      throw new Error('Setup failure');
+    }
+    // execute `%reset -f` cell
+    await layoutReady();
+    const minExpected = Math.min(...this.options.expectedNumberOfVariables);
+    await this.jupyterApp.commands.execute('notebook:run-cell-and-select-next');
+    // first two are "special variables" and "function variables"
+    await waitForKernelStatus(this.widget.node, 'idle');
+    await waitUntilDisappears(
+      `.jp-DebuggerVariables-body li:nth-child(${minExpected})`
+    );
+    await this._goToTop();
+    await layoutReady();
+  }
+}
+
+async function waitForKernelStatus(notebookPanel: HTMLElement, status: string) {
+  await new ElementHandle(notebookPanel).waitForSelector(
+    `.jp-Notebook-ExecutionIndicator[data-status="${status}"]`,
+    { state: 'attached' }
+  );
+}
+
 async function activateTabWidget(
   jupyterApp: JupyterFrontEnd,
   widget: MainAreaWidget
@@ -398,17 +554,12 @@ export class SwitchTabScenario implements IScenario {
     this._widgets = [];
     for (const tab of this._tabs) {
       let widget: MainAreaWidget;
-      switch (tab.type) {
-        case 'launcher':
-          widget = await this.jupyterApp.commands.execute('launcher:create');
-          break;
-        case 'file':
-          widget = await this.jupyterApp.commands.execute('docmanager:open', {
-            path: tab.path
-          });
-          break;
-        default:
-          throw Error('Unknown tab type');
+      if (tab.path) {
+        widget = await this.jupyterApp.commands.execute('docmanager:open', {
+          path: tab.path
+        });
+      } else {
+        widget = await this.jupyterApp.commands.execute('launcher:create');
       }
       await page.waitForSelector('#' + widget.id, { state: 'attached' });
       if (
