@@ -482,7 +482,8 @@ export function renderProfile(props: {
           typeof timing.resource === 'undefined' && timing.name === 'Profiler';
         const isOurProfilerCode =
           timing.resource &&
-          timing.resource.includes('@jupyterlab-benchmarks/ui-profiler');
+          (timing.resource.includes('@jupyterlab-benchmarks/ui-profiler') ||
+            timing.resource.includes('@jupyterlab/ui-profiler'));
         return !isNativeProfilerCall && !isOurProfilerCode;
       }
     );
@@ -725,6 +726,7 @@ export class BenchmarkHistory extends React.Component<
             ? 'up-BenchmarkHistory-file up-BenchmarkHistory-file-active'
             : 'up-BenchmarkHistory-file'
         }
+        key={file.name}
         onClick={() => {
           this.props.onSelect(file);
           this.setState({
@@ -764,7 +766,8 @@ function timingSummary(timing: ITimingOutcome): JSX.Element {
     <>
       <div
         title={
-          timing.reference.sort().map(Statistic.round).join(' ms\n') + ' ms'
+          [...timing.reference].sort().map(Statistic.round).join(' ms\n') +
+          ' ms'
         }
       >
         Reference: IQM:{' '}
@@ -936,17 +939,30 @@ export class BenchmarkMonitor extends React.Component<
             }
             let elapsed = NaN;
             let remaining = NaN;
-            let status: string;
+            let status = 'up-mod-waiting';
+
+            if (args.interrupted) {
+              status = 'up-mod-interrupted';
+            }
+            if (args.errored) {
+              status += ' up-mod-errored';
+            }
+
             if (this.start) {
               const now =
-                args.percentage === 100 && this.end ? this.end : new Date();
+                (args.percentage === 100 || args.interrupted || args.errored) &&
+                this.end
+                  ? this.end
+                  : new Date();
               this.end = now;
               elapsed = now.getTime() - this.start.getTime();
-              remaining = ((100 - args.percentage) * elapsed) / args.percentage;
-              status = args.percentage === 100 ? 'up-mod-completed' : '';
-            } else {
-              status = 'up-mod-waiting';
+              if (!args.interrupted && !args.errored) {
+                remaining =
+                  ((100 - args.percentage) * elapsed) / args.percentage;
+                status = args.percentage === 100 ? 'up-mod-completed' : '';
+              }
             }
+
             return (
               <div className={status + ' up-BenchmarkMonitor-content'}>
                 <div>
@@ -1020,6 +1036,7 @@ export class BenchmarkLauncher extends React.Component<
 > {
   constructor(props: ILauncherProps) {
     super(props);
+    this._stop = new Signal(this);
     this.state = {
       benchmarks: [props.benchmarks[0]],
       scenarios: [props.scenarios[0]],
@@ -1029,6 +1046,7 @@ export class BenchmarkLauncher extends React.Component<
       isRunning: false
     };
     this.runSelected = this.runSelected.bind(this);
+    this.stopCurrent = this.stopCurrent.bind(this);
   }
   state: IProfilerState;
 
@@ -1046,9 +1064,14 @@ export class BenchmarkLauncher extends React.Component<
     const result = (await benchmark.run(
       scenario,
       options.benchmark,
-      this.props.progress
+      this.props.progress,
+      this._stop
     )) as T;
-    this.props.progress.emit({ percentage: 100 });
+    if (result.interrupted) {
+      this.props.progress.emit({ percentage: NaN, interrupted: true });
+    } else {
+      this.props.progress.emit({ percentage: 100 });
+    }
     const data: Omit<IBenchmarkResultBase<T>, 'id'> = {
       result: result,
       options: options,
@@ -1117,6 +1140,11 @@ export class BenchmarkLauncher extends React.Component<
     this.setState({
       isRunning: true
     });
+    let stop = false;
+    const stopListener = () => {
+      stop = true;
+    };
+    this._stop.connect(stopListener);
     try {
       // copy to prevent user inadvertedly changing what is being run
       const scheduledBenchmarks = [...this.state.benchmarks];
@@ -1124,6 +1152,9 @@ export class BenchmarkLauncher extends React.Component<
       const config = JSONExt.deepCopy(this._config as any) as IConfigValue;
       for (const benchmark of scheduledBenchmarks) {
         for (const scenario of scheduledScenarios) {
+          if (stop) {
+            break;
+          }
           const result = await this.runBenchmark(scenario, benchmark, config);
           const filename = benchmarkFilename(result);
           this.props.upload(
@@ -1139,13 +1170,23 @@ export class BenchmarkLauncher extends React.Component<
         }
       }
     } catch (e) {
-      void showErrorMessage('Benchmark failed', e);
+      if (!stop) {
+        this.props.progress.emit({ percentage: NaN, errored: true });
+        void showErrorMessage('Benchmark failed', e);
+      } else {
+        this.props.progress.emit({ percentage: NaN, interrupted: true });
+      }
     }
+    this._stop.disconnect(stopListener);
     this.setState({
       isRunning: false
     });
   }
 
+  stopCurrent(): void {
+    this._stop.emit();
+  }
+
   render(): JSX.Element {
     const benchmarks = this.props.benchmarks.map(benchmark => {
       const disabled = benchmark.isAvailable ? !benchmark.isAvailable() : false;
@@ -1159,7 +1200,7 @@ export class BenchmarkLauncher extends React.Component<
         >
           <input
             type="checkbox"
-            checked={this.state.benchmarks.includes(benchmark)}
+            defaultChecked={this.state.benchmarks.includes(benchmark)}
             className="up-BenchmarkLauncher-choice-input"
             disabled={disabled}
             value={benchmark.id}
@@ -1173,7 +1214,7 @@ export class BenchmarkLauncher extends React.Component<
         <label key={scenario.id}>
           <input
             type="checkbox"
-            checked={this.state.scenarios.includes(scenario)}
+            defaultChecked={this.state.scenarios.includes(scenario)}
             className="up-BenchmarkLauncher-choice-input"
             value={scenario.id}
           />
@@ -1181,7 +1222,6 @@ export class BenchmarkLauncher extends React.Component<
         </label>
       );
     });
-    // TODO: stop button
     // TODO: custom widget for path selection, FileDialog.getOpenFiles
     return (
       <div className="up-BenchmarkLauncher" style={{ height: '450px' }}>
@@ -1262,7 +1302,10 @@ export class BenchmarkLauncher extends React.Component<
           <div className="up-BenchmarkLauncher-launchbar-buttons">
             <button
               onClick={this.runSelected}
-              className="jp-mod-styled jp-mod-accept"
+              className={
+                'jp-mod-styled jp-mod-accept' +
+                (this.state.isRunning ? ' jp-mod-hidden' : '')
+              }
               disabled={
                 this.state.scenarios.length === 0 ||
                 this.state.benchmarks.length === 0 ||
@@ -1271,6 +1314,16 @@ export class BenchmarkLauncher extends React.Component<
             >
               Start
             </button>
+            <button
+              onClick={this.stopCurrent}
+              className={
+                'jp-mod-styled jp-mod-warn' +
+                (!this.state.isRunning ? ' jp-mod-hidden' : '')
+              }
+              disabled={!this.state.isRunning}
+            >
+              Stop
+            </button>
           </div>
         </div>
       </div>
@@ -1281,4 +1334,6 @@ export class BenchmarkLauncher extends React.Component<
     scenarios: {},
     benchmarks: {}
   };
+
+  private _stop: Signal<BenchmarkLauncher, void>;
 }
